import {useEffect, useRef, useState} from "react";
import Swal from "sweetalert2";
import "./lobby.css";
import {useLocation, useNavigate, useParams} from "react-router-dom";
import { Quiz } from "../../../models/Quiz";
import { useMultiplayerService } from "../../../services/MultiplayerService";
import {toDataURL} from "qrcode";
import { User } from "../../../models/User";
import HttpError from "../../../services/HttpError.ts";
import {useEventSourceContext} from "../../../components/EventSourceContextType.tsx";



export function Lobby() {
    const { id } = useParams<{ id: string }>();
    const [quiz, setQuiz] = useState<Quiz>();
    const [players, setPlayers] = useState<User[]>([]);
    const [roomIdCreated, setRoomIdCreated] = useState<string | undefined>(id);
    const { setEventSource } = useEventSourceContext();
    const isFirstRender = useRef(true);
    const {createParty, startParty, generateRun} = useMultiplayerService();

    const navigate = useNavigate();
    const location = useLocation();
    const isHost = location.state?.isHost;

    let goodId = id;
    console.log("isHost :", isHost);
    const url = import.meta.env.VITE_API_URL;

    const initSSE = async () => {
        console.log("Init SSE for quiz ID:", id);
        if(isHost) {
            const response = await createParty(id);
            if (HttpError.isHttpError(response)) {
                console.error("Failed to create party:", response.message);
                return;
            }
            console.log("Created party with ID:", response);
            goodId = response;
            setRoomIdCreated(response);
        }
        console.log("User try to joined party with ID:", goodId);
        console.log(`${url}/party/join/${goodId}`)
        interface EventData extends Event {
            data: string; // EventSource envoie toujours les données en tant que string
        }
        const es = new EventSource(`${url}/party/join/${goodId}`,{
            withCredentials:true,
        });
        setEventSource(es);

        es.addEventListener("gameStarted",  (event: EventData) => {
            console.log("gameStarted listener")
            if(!event.data) return;
            console.log("Game started event data:",event.data);
            console.log("navigate to : /quizMulti/"+goodId);
            navigate("/quizMulti/"+goodId, { state: { questionCount: 10, isHost: isHost } });
        });

        es.addEventListener("userJoined", (event : EventData) => {
            console.log("User joined event data:", event.data);
            if(!event.data) return;

            const newUsers: User[] = JSON.parse(event.data);

            setPlayers((prevPlayers) => {
                const ids = new Set(prevPlayers.map((player) => player.id));
                const filteredNewUsers = newUsers.filter((user) => !ids.has(user.id));
                return [...prevPlayers, ...filteredNewUsers];
            });
        });

        es.addEventListener("error", (err) => {
            console.error("Error with SSE:", err);
        });

        return () => es.close();
    };

    useEffect(() => {
        if (isFirstRender.current) {
            isFirstRender.current = false;
            initSSE();
            return;
        }
    }, []);

    const leave = async () => {
        navigate("/multiplayer");
    };

    const showInvitation = async () => {
        const url =  window.location.href;
        const currentUrl = url.substring(0, url.lastIndexOf('/'))+"/"+roomIdCreated;
        const qrCodeUrl = await toDataURL(currentUrl);

        Swal.fire({
            title: "Invitation",
            html: `
                <p>Invite tes amis à jouer avec toi dans cette <part></part>ie !</p>
                <img src="${qrCodeUrl}" alt="QR Code" style="width: 200px; height: 200px; border-radius: 0%;" />
                <p>${currentUrl}</p>
            `,
            showCancelButton: true,
            confirmButtonText: "COPY LINK",
            cancelButtonText: "CLOSE",
        }).then((result) => {
            if (result.isConfirmed) {
                navigator.clipboard.writeText(currentUrl);
                Swal.fire("Copied!", "", "success");
            }
        });
    };

    const cancel = async () => {
        navigate("/multiplayer");
    };

    const play = async () => {
        console.log("start party in front");
        startParty();
    };

    return (
        <div className="container-right">
            <div className="left-side">
                <h1 className="title">PLAYERS</h1>
                <div className="player-list">
                    {players.map((player, index) => (
                        <div className="player" key={index}>
                            <img
                                src="https://cdn.pixabay.com/photo/2015/10/05/22/37/blank-profile-picture-973460_1280.png"
                                alt="profile picture"
                                className="player-image"
                            />
                            <p className="player-name">{player.username}</p>
                        </div>
                    ))}
                </div>
                <div className="new-quiz">
                    <button className="blueButton" onClick={showInvitation}>
                        INVITATION
                    </button>
                </div>
            </div>
            <div className="right-side">
                <div className="quiz-info">
                    <div className="quiz-title">
                        <p className="quiz-info-title">Quiz information</p>
                        <hr />
                    </div>

                    <div className="quiz-info-content">
                        <p className="quiz-info-content-value">{quiz?.category?.name}</p>
                    </div>
                    <div className="quiz-info-content">
                        {/*<p className="quiz-info-content-title">Nb. questions</p>*/}
                        <p className="quiz-info-content-value">{quiz?.questionCount}</p>
                    </div>
                </div>

                <div className="new-quiz">
                    {!isHost ? (
                        <button className="blueButton" onClick={leave}>
                            LEAVE
                        </button>
                    ) : (
                        <div className="host-buttons">
                            <button className="blueButton" onClick={play}>
                                PLAY
                            </button>
                            <button className="whiteButton" onClick={cancel}>
                                CANCEL
                            </button>
                        </div>
                    )}
                </div>
            </div>
        </div>
    );
}
