import "./quizBot.css";
import { AnswerButton } from "../../../../components/quiz/answerButton/answerButton.tsx";
import { ValidationButton } from "../../../../components/quiz/validationButton/validationButton.tsx";
import { useEffect, useState } from "react";
import { useMultiplayerService } from "../../../../services/MultiplayerService.ts";
import { Answer } from "../../../../models/Answer.ts";
import {Question} from "../../../../models/Question.ts";
import {AnswerSSEResponse} from "../../../../models/Response/AnswerSSEResponse.ts";
import {useEventSourceContext} from "../../../../components/EventSourceContextType.tsx";
import {PlayingQuizState} from "../../../../models/PlayingQuizState.ts";
import HttpError from "../../../../services/HttpError.ts";

interface Props {
    actualQuestion: Question;
    isHost: boolean;
    setPlayingQuizState: (playingQuizState: PlayingQuizState) => void;
    playingQuizState: PlayingQuizState;
    OpenModalEndQuiz: (bool: boolean) => void;
    setScore: (score: number) => void;
}


const getStyleOfAnswerButton = (
    answerId: number,
    selectedAnswerIndex: number | null,
    correctAnswerId: number | null,
    quizState: PlayingQuizState
): string => {
    if (quizState === PlayingQuizState.LOADING) {
        return "answer-button answer-button-loading";
    }
    if (quizState === PlayingQuizState.ANSWERING) {
        if (selectedAnswerIndex === null) return "answer-button";
        if (selectedAnswerIndex === answerId) return "answer-button answer-button-selected";
    }
    if (quizState === PlayingQuizState.SHOWING_RESULTS) {
        if (answerId === correctAnswerId) return "answer-button answer-button-correct";
        if (answerId !== correctAnswerId && selectedAnswerIndex === answerId)
            return "answer-button answer-button-wrong";
    }
    return "answer-button";
};
export function QuizBotMulti({ actualQuestion, isHost, playingQuizState, setPlayingQuizState, setScore }: Props) {
    const [questionIsFinished, setQuestionIsFinished] = useState(false);
    const [selectedAnswerId, setSelectedAnswerId] = useState<number | null>(null);
    const [validationButtonIsDisabled, setValidationButtonIsDisabled] = useState(true);
    const [correctAnswerId, setCorrectAnswerId] = useState<number | null>(null);
    const { nextQuestion, answerQuestion, revealAnswer, getScore } = useMultiplayerService();
    const { eventSource } = useEventSourceContext();

    const initSSE = async () => {
        eventSource.addEventListener("answerRevealed", (event) => {
            console.log("answerRevealed");
            if(!event.data) return;
            const answers: AnswerSSEResponse = JSON.parse(event.data);
            setCorrectAnswerId(getCorrectAnswer(answers.answers).id);
            setPlayingQuizState(PlayingQuizState.SHOWING_RESULTS);
        });
    }

    useEffect(() => {
        initSSE();
    }, []);

    const getCorrectAnswer = (answers: Answer[]): Answer => {
        const correctAnswer = answers.find((answer) => answer.isCorrect);
        if (!correctAnswer) throw new Error("No correct answer found");
        return correctAnswer;
    };
    const onValidation = async () => {
        console.log("onValidation");
        if(selectedAnswerId === null || actualQuestion === undefined) return;
        setPlayingQuizState(PlayingQuizState.LOADING);
        await answerQuestion(actualQuestion.id, selectedAnswerId);

        if(isHost)
        {
            await revealAnswer();
        }
        const scoreFetched = await getScore();
        if(HttpError.isHttpError(scoreFetched))
        {
            return;
        }
        setScore(scoreFetched);

    };

    const onContinue = () => {
        if(isHost)
        {
            nextQuestion();
            setPlayingQuizState(PlayingQuizState.ANSWERING);
            setCorrectAnswerId(null);
            setSelectedAnswerId(null);
        }
    };

    const onAnsweredButtonClicked = (answerId: number) => {
        if (playingQuizState === PlayingQuizState.SHOWING_RESULTS || playingQuizState === PlayingQuizState.LOADING) {
            return;
        }
        setSelectedAnswerId(answerId);
    };

    return (
        <div className="quiz-bot-container">
            <div className="quiz-bot-button-container">
                {actualQuestion.answers.map((answer) => {
                    return (
                        <>
                            {/*<p>{quizState === QuizState.SHOWING_RESULTS ? answer.nb_answers : null}</p>*/}
                            <AnswerButton
                                text={answer.text}
                                key={answer.id}
                                onClick={() => onAnsweredButtonClicked(answer.id)}
                                style={getStyleOfAnswerButton(answer.id, selectedAnswerId, correctAnswerId, playingQuizState)}
                                isDisabled={playingQuizState === PlayingQuizState.LOADING}
                            />
                        </>
                    );
                })}
            </div>
            <div className="quiz-bot-validation-button-container">
                <ValidationButton
                    text={playingQuizState === PlayingQuizState.ANSWERING ? "CONFIRM" : "CONTINUE"}
                    onClick={() => (playingQuizState === PlayingQuizState.ANSWERING ? onValidation() : onContinue())}
                    isDisabled={playingQuizState !== PlayingQuizState.ANSWERING && !isHost}
                />
            </div>
        </div>
    );
}
